/*
 * originally written by Lars Wirzenius: read a line into memory,
 * with no max length. Return a pointer to the line, or NULL in case
 * of error. Strip \r at EOL
 * redone for NUL bytes in string by Cornelius Krasel
 *
 * getaline() will not return anything if it gets a single NUL byte
 * to read. This is not dramatic since it should not occur in NNTP
 * traffic anyway.
 */

#include "leafnode.h"
#include <string.h>
#include <stdio.h>
#include <syslog.h>

char * getaline(FILE *f) {
    static char *buf;       /* buffer for line */
    size_t size;	    /* size of buffer */
    size_t len;             /* # of chars stored into buf before '\0' */
    char * p;

    len  = 0;
    size = 256;		/* to conserve memory;
			 * otherwise we may generate a lot of huge buffers */
    if (!buf) {
	buf = critmalloc( size, "reading line" );
    }
    else {
	buf = critrealloc( buf, size, "reading line" );
    }

    /*
     * for the nntp protocol, a line is defined as something terminating
     * with \n
     *
     * unfortunately some programs, e.g. DNEWS, send NUL bytes over
     * NNTP sometimes, making it difficult to properly read a line.
     * Therefore, the end of a line is determined if the end of the
     * fgets() string is \n\0 (with the \0 appended by fgets anyway).
     * If an upstream sends just \0 inside of a string, it is ignored.
     */
    while ((p=fgets(buf+len, size-len, f)) != NULL) {
	if (*(buf+len) == '\0')
	    break;		/* don't throw up on NUL bytes */
	len += strlen(buf+len);
	if ( buf[len-1] == '\n' )
	    break;		/* the whole line has been read */

	size += size;		/* exponential growth of buffer */
	buf = critrealloc(buf, size, "reading line" );
    }

    if ( len == 0 )
	return NULL;

    if (len && (buf[len-1] == '\n')) { /* go back on top of the newline */
	--len;
	if (len && (buf[len-1] == '\r')) /* also delete CR */
	    --len;
    }

    buf[len] = '\0';        /* unconditionally terminate string,
                               possibly overwriting newline */

    if ( debug > 1 )
        syslog( LOG_DEBUG, "<%s\n", buf );
    return buf;
}

